function [numErr, numErrACI, numSyms] = commPassbandModPCT(nTx, EbNoVect, idx)
%COMMPASSBANDMODPCT Simulates a 16-QAM passband communications system
%for a number of frame transmissions for a single Eb/No value.
%
% [NUMERR, NUMERRACI, NUMSYMS] = COMMPASSBANDMODPCT(NTX, EBNOVECT,
% IDX) simulates a passband communications system using 16-QAM baseband
% modulation objects and frequency up and downconverters, and using an
% additive white Gaussian noise channel with interference in the passband
% caused by a tone in an adjacent channel that has been processed by a
% cubic nonlinearity. Simulations are performed for a number of frame
% transmissions specified by NTX for a single Eb/No value. The function
% returns the number of errors, NUMERR, for the passband simulation without
% adjacent channel interference, and the number of errors NUMERRACI, for
% the passband simulation with adjacent channel interference and the total
% number of symbols NUMSYMS. EBNOVECT is the Eb/No range and IDX is the
% current Eb/No value index.

% Copyright 2014 The MathWorks, Inc.

%% Initialize
Fc = 2.5e6;                % Carrier frequency (Hz)
Rsym = 1e6;                % Symbol rate (symbols/second)
nSamps = 8;                % Number of samples per symbol
Fs = Rsym * nSamps;        % Sampling frequency (Hz)
frameLength = 5000;        % Number of symbols in a frame
M = 16;                    % Modulation order (16-QAM)
Fint = Fc/3+50e3;          % Tonal interference frequency
Aint = 0.45;               % Tonal interference amplitude
%% Setup

% Create a 16-QAM modulator and demodulator.
hMod = modem.qammod('M',M);
hDemod = modem.qamdemod(hMod);

% Pulse shaping: specify a square root raised cosine filter with a filter length
% of eight symbols and a rolloff factor of 0.2.
nSym = 8;   % Length of the filter in symbols
beta = 0.2; % Rolloff factor
filterSpec = fdesign.pulseshaping(nSamps, 'Square root raised cosine', ...
    'Nsym,Beta', nSym, beta);
hXmtFlt = design(filterSpec);
hXmtFlt.PersistentMemory = true;
hRcvFlt = copy(hXmtFlt);
hRcvFltACI = copy(hXmtFlt);

% Compute signal power.
avgPwrBaseBand16QAM = 10*log10(sum(abs(hMod.Constellation).^2)/(M*nSamps));
sigPower = 10*log10(sum(hXmtFlt.Numerator.^2)) + avgPwrBaseBand16QAM;

%% Simulation

% Initialize error counters and loop over the desired number of transmissions
% and for all EbNo values.
numErr = zeros(length(EbNoVect),1);
numErrACI = zeros(length(EbNoVect),1);

tStart = 0;
SNR = EbNoVect(idx) + 10*log10(log2(M)/nSamps) + 10*log10(2);

for p = 1:nTx
    b = randi([0 hMod.M-1], frameLength, 1);
    txSym = modulate(hMod, b);
    
    % Pulse shaping.
    x = filter(hXmtFlt, upsample(txSym, nSamps));
    
    % Upconvert to passband.
    tEnd = (tStart+frameLength/Rsym)-1/Fs;
    t = (tStart:1/Fs:tEnd).';
    tStart = tEnd;
    carrier = sqrt(2)*exp(1i*2*pi*Fc*t);
    xUp = real(x.*carrier);
    
    % Add AWGN.
    yUp = awgn(xUp, SNR, sigPower);
    
    % Add adjacent channel interference.
    interference = Aint*cos(2*pi*Fint*t+pi/8).^3;
    yUpACI = yUp + interference;
    
    % Downconvert signals with and without adjacent channel interference
    % (Assumes perfect synchronization).
    y = yUp.*conj(carrier);
    yACI = yUpACI.*conj(carrier);
    
    % Apply matched filter.
    rcvSym = filter(hRcvFlt, y);
    rcvSymACI = filter(hRcvFltACI, yACI);
    
    % Amplify the received signal symbols to place them around the center of
    % the 16-QAM constellation points.
    rcvSym = nSym*rcvSym;
    rcvSymACI = nSym*rcvSymACI;
    
    % Downsample the signals.
    e2eDelay = nSamps*nSym;
    rcvSymDown = rcvSym(e2eDelay+1:nSamps:end);
    rcvSymDownACI = rcvSymACI(e2eDelay+1:nSamps:end);
    
    % Demodulate.
    xHat = demodulate(hDemod, rcvSymDown);
    xHatACI = demodulate(hDemod, rcvSymDownACI);
    
    % Compute number of symbol errors.
    numErr(idx) = numErr(idx) + sum(xHat~=b(1:end-nSym));
    numErrACI(idx) = numErrACI(idx) + sum(xHatACI~=b(1:end-nSym));
end

% Reset the filters before starting simulations for a new SNR value.
reset(hXmtFlt);
reset(hRcvFlt);
reset(hRcvFltACI);

numSyms = (frameLength*nTx);
